/**********************************************************************
 * Copyright (c) 2003, 2011 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: XMLoptionLoader.java,v 1.2 2007-12-01 00:06:25 apnan Exp $
 *
 * Contributors:
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.loaders.hierarchy;

import org.eclipse.hyades.loaders.util.HierarchyContext;
import org.eclipse.hyades.loaders.util.LoadersUtils;
import org.eclipse.hyades.models.hierarchy.HierarchyFactory;
import org.eclipse.hyades.models.hierarchy.TRCAgent;
import org.eclipse.hyades.models.hierarchy.TRCAgentProxy;
import org.eclipse.hyades.models.hierarchy.TRCConfiguration;
import org.eclipse.hyades.models.hierarchy.TRCOption;
import org.eclipse.hyades.models.hierarchy.TRCTIProfilerCollectionMode;
import org.eclipse.hyades.models.hierarchy.util.EMFUtil;


public class XMLoptionLoader extends IgnoredXMLFragmentLoader {
    //~ Instance fields ----------------------------------------------------------------------------

    protected String key;
    protected String optionValue;

    //~ Methods ------------------------------------------------------------------------------------

    /* (non-Javadoc)
     * @see org.eclipse.hyades.loaders.util.XMLFragmentLoader#addAttribute(java.lang.String, java.lang.String)
     */
    public void addAttribute(String name, String value) {
        if (name.equals("key")) {
            key = value;
        } else if (name.equals("value")) {
            optionValue = value;
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.hyades.loaders.util.XMLFragmentLoader#addYourselfInContext(org.eclipse.hyades.loaders.util.HierarchyContext)
     */
    public void addYourselfInContext() {
        TRCAgent agent = context.getAgent();

        if (agent == null) {
            LoadersUtils.createAgent(context);
        }

        TRCAgentProxy agentProxy = context.getAgentProxy();
		if (agentProxy == null) {
			agentProxy = agent.getAgentProxy();
		}

        TRCConfiguration config = null;
        Object[] configList = agentProxy.getConfigurations().toArray();

        for (int idx = 0; idx < configList.length; idx++) {
            TRCConfiguration c = (TRCConfiguration) configList[idx];

            if ((c.getName() != null) && c.getName().equals("__JVMPI__")) {
                config = c;

                break;
            }
        }

        if (config == null) {
            config = HierarchyFactory.eINSTANCE.createTRCConfiguration();
            config.setName("__JVMPI__");
            config.setActive(true);
            config.setAgentProxy(agentProxy);
        }

        TRCOption option = HierarchyFactory.eINSTANCE.createTRCOption();
        
        option.setKey(key);
        option.setValue(optionValue);
        
        // If the current option can be used to determine the type of profiling we are doing,
        // then use it to set the TI collection value
        if(key != null && optionValue != null && agent != null && agent.getAgentProxy() != null) {
        	if(agent.getAgentProxy().getTiCollectionMode() == null || agent.getAgentProxy().getTiCollectionMode() == TRCTIProfilerCollectionMode.UNKNOWN) {
        		
        		TRCTIProfilerCollectionMode mode = calcTICollectionMode(option);
        		if(mode != null && mode != TRCTIProfilerCollectionMode.UNKNOWN) {
        			agent.getAgentProxy().setTiCollectionMode(mode);
        		}
        	}
        	
        }
        

        if(Constants.MULTIPLE_HEAP_DUMPS.equals(key))
        {
        	if(Boolean.valueOf(optionValue).booleanValue())
        		context.getCustomData().put(key,Boolean.TRUE);
        	else
        		context.getCustomData().remove(key);
        }
        else
        if(Constants.LLDATA_ENABLED.equals(key))
        {
        	if(Boolean.valueOf(optionValue).booleanValue())
        		context.getCustomData().put(key,Boolean.TRUE);
        	else
        		context.getCustomData().remove(key);
        }
        	
        if(!EMFUtil.containsObjectUsingSpecialEqual(option,config))
        {
            option.setConfiguration(config);
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.hyades.loaders.util.XMLFragmentLoader#initialize(java.lang.String)
     */
    public void initialize(HierarchyContext context, String name) {
        super.initialize(context, name);
        key = null;
        optionValue = null;
    }
    
	private static TRCTIProfilerCollectionMode calcTICollectionMode(TRCOption o) {
		boolean isHeap = false;
		boolean isHeapObjAllocSites = false;

		boolean isExec = false;
		boolean isExecFlow = false;

		boolean isThread = false;

		if (o.getKey().equalsIgnoreCase("org.eclipse.tptp.platform.jvmti.client.ATTR_HEAP_COLLECT_INSTANCE")) { //$NON-NLS-1$
			isHeap = true;

		}
		if (o.getKey().equalsIgnoreCase("org.eclipse.tptp.platform.jvmti.client.ATTR_EXEC_OBJ_ALLOC_SITE")) { //$NON-NLS-1$
			isHeapObjAllocSites = (o.getValue() != null ? o.getValue().trim().equalsIgnoreCase("true") : false); //$NON-NLS-1$
		}

		if (o.getKey().equalsIgnoreCase("org.eclipse.tptp.platform.jvmti.client.ATTR_EXEC_DATA")) { //$NON-NLS-1$
			isExec = true;
		}

		if (o.getKey().equalsIgnoreCase("org.eclipse.tptp.platform.jvmti.client.ATTR_EXEC_FLOW")) { //$NON-NLS-1$
			isExecFlow = (o.getValue() != null ? o.getValue().trim().equalsIgnoreCase("true") : false); //$NON-NLS-1$
		}
		
		if(o.getKey().equalsIgnoreCase("org.eclipse.tptp.platform.jvmti.client.ATTR_EXEC_CONTENTION_ANALYSIS")) { //$NON-NLS-1$
			isThread = true;
		}

		if (isExec) {
			if (isExecFlow) {
				return TRCTIProfilerCollectionMode.EXEC_FLOW;
			} else {
				return TRCTIProfilerCollectionMode.EXEC_STATS;
			}
		} else if (isHeap) {
			if (isHeapObjAllocSites) {
				return TRCTIProfilerCollectionMode.HEAP_ALLOC_SITES;
			} else {
				return TRCTIProfilerCollectionMode.HEAP_NO_ALLOC_SITES;
			}
		} else if (isThread) {
			return TRCTIProfilerCollectionMode.THREAD_CONTENTION;
		}
		
		return null;

	}
    
}
