/**********************************************************************
 * Copyright (c) 2005, 2012 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: AbstractProfileActionDelegate.java,v 1.5 2008-01-24 02:29:59 apnan Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.trace.ui.actions;

import java.util.Iterator;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.tptp.platform.common.ui.trace.internal.CommonUITraceConstants;
import org.eclipse.tptp.platform.common.ui.trace.internal.CommonUITracePlugin;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

public abstract class AbstractProfileActionDelegate implements IWorkbenchWindowActionDelegate, IViewActionDelegate, ISelectionListener
{

	/**
	 * The underlying action for this delegate
	 */
	protected IAction fAction;

	/**
	 * This action's view part, or <code>null</code> if not installed in a
	 * view.
	 */
	protected IViewPart fViewPart;

	/**
	 * Cache of the most recent selection
	 */
	protected IStructuredSelection fSelection;

	/**
	 * Whether this delegate has been initialized
	 */
	protected boolean fInitialized = false;

	/**
	 * It's crucial that delegate actions have a zero-arg constructor so that
	 * they can be reflected into existence when referenced in an action set in
	 * the plugin's plugin.xml file.
	 */
	public AbstractProfileActionDelegate()
	{
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#dispose()
	 */
	public void dispose()
	{
		CommonUITracePlugin.getDefault().getWorkbench().getActiveWorkbenchWindow().getSelectionService().removeSelectionListener(CommonUITraceConstants.ID_PD_NAVIGATOR_VIEW, this);

		fAction = null;
		fViewPart = null;
		fSelection = null;

	}

	/**
	 * @see IWorkbenchWindowActionDelegate#init(IWorkbenchWindow)
	 */
	public void init(IWorkbenchWindow window)
	{
		// listen to selection changes in the profile view
		window.getSelectionService().addSelectionListener(CommonUITraceConstants.ID_PD_NAVIGATOR_VIEW, this);
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	@SuppressWarnings("rawtypes")
	public void run(IAction action)
	{
		IStructuredSelection selection = getSelection();

		final Iterator enum1 = selection.iterator();
		while (enum1.hasNext())
		{
			Object element = enum1.next();
			try
			{
				doAction(element);
			} catch (Exception e)
			{
				e.printStackTrace();
			}
		}

	}

	/**
	 * Set the icons for this action on the first selection changed event. This
	 * is necessary because the XML currently only supports setting the enabled
	 * icon.
	 * <p>
	 * AbstractProfileActionDelegates come in 2 flavors: IViewActionDelegate,
	 * IWorkbenchWindowActionDelegate delegates.
	 * </p>
	 * <ul>
	 * <li>IViewActionDelegate delegate: getView() != null</li>
	 * <li>IWorkbenchWindowActionDelegate: getView == null</li>
	 * </ul>
	 * <p>
	 * Only want to call update(action, selection) for IViewActionDelegates. An
	 * initialize call to update(action, selection) is made for all flavors to
	 * set the initial enabled state of the underlying action.
	 * IWorkbenchWindowActionDelegate's listen to selection changes in the debug
	 * view only.
	 * </p>
	 * 
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection s)
	{
		boolean wasInitialized = initialize(action, s);
		if (!wasInitialized)
		{
			if (getView() != null)
			{
				update(action, s);
			}
		}
	}

	protected void update(IAction action, ISelection s)
	{
		if (action == null || s == null)
			return;
		if (s instanceof IStructuredSelection)
		{
			IStructuredSelection ss = (IStructuredSelection) s;
			action.setEnabled(getEnableStateForSelection(ss));
			setSelection(ss);
		} else
		{
			action.setEnabled(false);
			setSelection(StructuredSelection.EMPTY);
		}
	}

	/**
	 * Return whether the action should be enabled or not based on the given
	 * selection.
	 */
	@SuppressWarnings("rawtypes")
	protected boolean getEnableStateForSelection(IStructuredSelection selection)
	{
		if (selection.size() == 0)
		{
			return false;
		}
		Iterator enum1 = selection.iterator();
		int count = 0;
		while (enum1.hasNext())
		{
			count++;
			if (count > 1 && !enableForMultiSelection())
			{
				return false;
			}
			Object element = enum1.next();
			if (!(isEnabledFor(element)))
			{
				return false;
			}
		}
		return true;
	}

	/**
	 * Returns whether this action should be enabled if there is multi
	 * selection.
	 */
	protected boolean enableForMultiSelection()
	{
		return true;
	}

	/**
	 * Does the specific action of this action to the process.
	 */
	protected abstract void doAction(Object element) throws Exception;

	/**
	 * Returns whether this action will work for the given element
	 */
	protected abstract boolean isEnabledFor(Object element);

	/**
	 * @see IViewActionDelegate#init(IViewPart)
	 */
	public void init(IViewPart view)
	{
		fViewPart = view;
	}

	/**
	 * Returns this action's view part, or <code>null</code> if not installed
	 * in a view.
	 * 
	 * @return view part or <code>null</code>
	 */
	protected IViewPart getView()
	{
		return fViewPart;
	}

	/**
	 * Initialize this delegate, updating this delegate's presentation. As well,
	 * all of the flavors of AbstractDebugActionDelegates need to have the
	 * initial enabled state set with a call to update(IAction, ISelection).
	 * 
	 * @param action
	 *            the presentation for this action
	 * @return whether the action was initialized
	 */
	protected boolean initialize(IAction action, ISelection selection)
	{
		if (!isInitialized())
		{
			setAction(action);
			update(action, selection);
			setInitialized(true);
			return true;
		}
		return false;
	}

	/**
	 * Returns the most recent selection
	 * 
	 * @return structured selection
	 */
	protected IStructuredSelection getSelection()
	{
		return fSelection;
	}

	/**
	 * Sets the most recent selection
	 * 
	 * @parm selection structured selection
	 */
	protected void setSelection(IStructuredSelection selection)
	{
		fSelection = selection;
	}

	/**
	 * @see ISelectionListener#selectionChanged(IWorkbenchPart, ISelection)
	 */
	public void selectionChanged(IWorkbenchPart part, ISelection selection)
	{
		update(getAction(), selection);
	}

	protected void setView(IViewPart viewPart)
	{
		fViewPart = viewPart;
	}

	protected boolean isInitialized()
	{
		return fInitialized;
	}

	protected void setInitialized(boolean initialized)
	{
		fInitialized = initialized;
	}

	protected void setAction(IAction action)
	{
		fAction = action;
	}

	protected IAction getAction()
	{
		return fAction;
	}

}
