/*****************************************************************************
 * Copyright (c) 2012 IBM.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    IBM - Initial API and implementation
 *
 * $Id$
 *****************************************************************************/

#ifdef EM64T_ARCH
#pragma runtime_checks( "", off )
#endif

#include <assert.h>
#if defined(MARTINI_OSAW_TRACE_RWLOCK) || defined(MARTINI_OSAW_PERFSTATS_RWLOCK)
#	include <iostream>
#endif
#include "OSAW.h"

//
// Since this is a home-brew RW-lock its worth documenting the implementation strategy:
//
// First of all the reason that a home-brew lock is being used instead of a 
// straight-up semaphore is because semaphores are kernel-level objects and
// _always_ incur a system call when releasing or signalling. The primary use
// of Martini::OSA::IRWLock is CGProf which must acquire a read-lock for
// every method call in the profiling filter set. A system call per method
// invocation is far too high a price to pay. Hence a userland solution is 
// necessary.
//
// Userland RW-locks are only available in Windows Vista and later; since we
// must support at least XP we are left with implementing our own solution.
//
// The lock is composed of three primitives:
//
//	DWORD m_owner - ID of owning thread when the lock has been acquired 
//                  for writing; 0 otherwise.
//  LONG m_count - Records the state of contention. A value of 0 indicates
//                 the lock is open. A positive value indicates the number
//                 of readers that have acquired the lock. A negative 
//                 value indicates that the lock has been acquired for
//                 writing; in this case, the absolute value indicates the
//                 number of times the lock has recursively been acquired.
//  HANDLE m_sem - A handle to an OS-level Semaphore object. Contended 
//                 acquisitions use the semaphore to be notified when the lock
//                 has been released.
//
// Acquires are implemented by attempting to update m_count according to
// the type of access requested. If the update fails (is contended by another
// thread) then we wait to be signaled via m_sem. Releases are handled by 
// performing the inverse of the acquire.
//
// Properties of this implementation:
//
//  - If a read-lock is requested by the thread which already owns the 
//    write-lock then the read-lock is implemented as a recursive write-lock.
//
//    This introduces a particular behavioural subtlety:
//
//     if recursive acquires are taken, they must be released in the reverse 
//     order they are taken (e.g. the acquire sequence write-read-read must
//     be released by the sequence read-read-write); anything different is
//     usually not intended anyway
//    
//    In exchange for this constraint, we avoid the complicated logic of 
//    maintaining separate read and write counters.
//
// - Because ownership of readers is not tracked, a deadlock will occur
//   if a thread attempts to obtain a write-lock when it has already obtained
//   a read-lock
//
// - The semaphore has only 1 'slot'. If multiple threads are waiting on the 
//   lock, then the signal will likely only wake up one thread at a time.
//
//   No issues have been seen so far, but this has implications for fairness.
//
//   In practice this lock should only be used when write-contention is very 
//   low compared with read-contention. In this case semaphore waits are very
//   rare and fairness shouldn't be an issue.
//  
// - In debug builds we aim for fail-fast behaviour by aggressively enforcing
//   all our known invariants via assert(). In release builds no invariants are
//   checked and improper use will likely result in deadlocks, hangs or other
//   undesirable behaviour.
//

#if defined(MARTINI_OSAW_PERFSTATS_RWLOCK)

static volatile LONG maxSpincount = 0;
static volatile LONG acquireSpincount = 0;
static volatile LONG releaseSpincount = 0;
static volatile LONG readerAcquires = 0;
static volatile LONG writerAcquires = 0;
static volatile LONG readerWaits = 0;
static volatile LONG writerWaits = 0;

static volatile LONG perfstatsInit = 0;

static void dumpPerfstats() {

	std::cerr << std::endl;
	std::cerr << "Martini::OSA::CRWLock aggregate stats:" << std::endl;
	std::cerr << "  max spincount   : " << maxSpincount << std::endl;
	std::cerr << "  acquire spins   : " << acquireSpincount << std::endl;
	std::cerr << "  release spins   : " << releaseSpincount << std::endl;
	std::cerr << "  avg. acq spins  : " << ( double(acquireSpincount)/double(readerAcquires+writerAcquires) ) << std::endl;
	std::cerr << "  avg. rel spins  : " << ( double(releaseSpincount)/double(readerAcquires) ) << std::endl;
	std::cerr << std::endl;
	std::cerr << "  reader acquires : " << readerAcquires << std::endl;
	std::cerr << "  writer acquires : " << writerAcquires << std::endl;
	std::cerr << "  reader waits    : " << readerWaits << std::endl;
	std::cerr << "  writer waits    : " << writerWaits << std::endl;
	std::cerr << "  read contention : " << ( double(readerWaits)/double(readerAcquires) ) << std::endl;
	std::cerr << "  write contention: " << ( double(writerWaits)/double(writerAcquires) ) << std::endl;
	
	std::cerr << std::endl;

}

#endif

Martini::OSA::CRWLock::CRWLock() : m_semaphore(INVALID_HANDLE_VALUE), m_count(0), m_owner(0)
{
}

Martini::OSA::CRWLock::~CRWLock()
{
	CloseHandle( m_semaphore );
}

#define MAX_SPINCOUNT 0x0fffffff

#ifdef MARTINI_OSAW_TRACE_RWLOCK

void Martini::OSA::CRWLock::trace( const char *method ) {

	int sp = 0; // hack-ish way to get a measure of the stack pointer; simple way to determine stack depth in the trace
	std::cerr << "(" << GetCurrentThreadId() << ", sp=" << (&sp) << ") Martini::OSA::CRWLock." << method << ": {"
		<< this << ", " << m_count << ", " << m_owner << "}" << std::endl;

}

#endif

TResult Martini::OSA::CRWLock::EnterReader()
{
#ifdef MARTINI_OSAW_TRACE_RWLOCK
	trace( "EnterReader" );
#endif

	LONG spincount = 0;
	while( true ) {

		// Something is probably wrong if this happens
		assert( spincount < MAX_SPINCOUNT );

		// If we already own a write lock, treat this as a recursive write-acquire
		DWORD owner = m_owner;
		if( GetCurrentThreadId() == owner )
			return EnterWriter();

		assert( owner != GetCurrentThreadId() );

		LONG state = m_count;
		// Owned by a writer? Wait on the semaphore
		if( state < 0 ) {

			WaitForSingleObject( m_semaphore, 0 );
#if defined(MARTINI_OSAW_PERFSTATS_RWLOCK)
			InterlockedIncrement( &readerWaits );
#endif
			spincount++;
			continue;

		// Un-owned, bump reader count
		} else if ( state == InterlockedCompareExchange(&m_count, state+1, state) ) {

			assert( m_count > 0 );
			break;

		}
		spincount++; 

	}

#if defined(MARTINI_OSAW_PERFSTATS_RWLOCK)
	InterlockedIncrement( &readerAcquires );
	while( spincount > maxSpincount )
		InterlockedCompareExchange(&maxSpincount, spincount, maxSpincount);
	while( true ) {
		LONG oldSpincount = acquireSpincount;
		if( oldSpincount == InterlockedCompareExchange(&acquireSpincount, acquireSpincount+spincount, oldSpincount) )
			break;
	}
#endif
	return MRTE_RESULT_OK;
}

TResult Martini::OSA::CRWLock::LeaveReader()
{
#ifdef MARTINI_OSAW_TRACE_RWLOCK
	trace( "LeaveReader" );
#endif

	LONG spincount = 0;
	while( true ) {

		assert( spincount < MAX_SPINCOUNT );

		// If we own the write lock, then we treat this as a recursive write-release
		DWORD owner = m_owner;
		if( GetCurrentThreadId() == owner )
			return LeaveWriter();

		assert( 0 == owner );

		LONG state = m_count;
		assert( state > 0 );

		if( state == InterlockedCompareExchange(&m_count, state-1, state) ) {
			// We are the last reader to relinquish the lock; signal waiting writers
			if( 1 == state )
				ReleaseSemaphore( m_semaphore, 1, NULL );
			break;
		}
		spincount++;

	}
#if defined(MARTINI_OSAW_PERFSTATS_RWLOCK)
	while( spincount > maxSpincount )
		InterlockedCompareExchange(&maxSpincount, spincount, maxSpincount);
	while( true ) {
		LONG oldSpincount = releaseSpincount;
		if( oldSpincount == InterlockedCompareExchange(&releaseSpincount, releaseSpincount+spincount, oldSpincount) )
			break;
	}
#endif
	return MRTE_RESULT_OK;
}

TResult Martini::OSA::CRWLock::EnterWriter()
{
#ifdef MARTINI_OSAW_TRACE_RWLOCK
	trace( "EnterWriter" );
#endif

	LONG spincount = 0;
	while( true ) {

		assert( spincount < MAX_SPINCOUNT );

		LONG state = m_count;

		// We already own the lock; perform a recursive acquire
		if( GetCurrentThreadId() == m_owner ) {

			if( state != InterlockedCompareExchange(&m_count, state-1, state) )
				// This would mean someone else has updated m_count even
				// though we own the lock. This is a bug.
				assert( 0 && "RWLock constraints violated!" );
			break;

		}
		// Try to acquire for writing
		else if( state != InterlockedCompareExchange(&m_count, -1, 0) ) {

			// Contended, wait on the semaphore
			WaitForSingleObject( m_semaphore, 0 );
#if defined(MARTINI_OSAW_PERFSTATS_RWLOCK)
			InterlockedIncrement( &writerWaits );
#endif
			spincount++;
			continue;

		}

		// Acquired! Store our ownership ID
		assert( 0 == m_owner );
		m_owner = GetCurrentThreadId();
		break;

	}
#if defined(MARTINI_OSAW_PERFSTATS_RWLOCK)
	InterlockedIncrement( &writerAcquires );
	while( spincount > maxSpincount )
		InterlockedCompareExchange(&maxSpincount, spincount, maxSpincount);
	while( true ) {
		LONG oldSpincount = acquireSpincount;
		if( oldSpincount == InterlockedCompareExchange(&acquireSpincount, acquireSpincount+spincount, oldSpincount) )
			break;
	}
#endif
	return MRTE_RESULT_OK;
}

TResult Martini::OSA::CRWLock::LeaveWriter()
{
#ifdef MARTINI_OSAW_TRACE_RWLOCK
	trace( "LeaveWriter" );
#endif
	assert( GetCurrentThreadId() == m_owner );

	LONG state = m_count;
	assert( state < 0 );

	if( state == InterlockedCompareExchange(&m_count, state+1, state) ) {

		assert( GetCurrentThreadId() == m_owner );

		// This is the outermost acquire; release the lock and notify
		if( -1 == state ) {
			m_owner = 0;
			ReleaseSemaphore( m_semaphore, 1, NULL );
		}

	}
	else {

		// This would mean someone else has updated m_count even
		// though we own the lock. This is a bug.
		assert( 0 && "RWLock constraints violated: m_count altered outside of owning thread!" );

	}
	return MRTE_RESULT_OK;
}

TResult Martini::OSA::CRWLock::Destroy()
{
	delete this;
	return MRTE_RESULT_OK;
}

TResult Martini::OSA::CRWLock::Create()
{
#if defined(MARTINI_OSAW_PERFSTATS_RWLOCK)
	while( !perfstatsInit ) {
		if( 0 == InterlockedCompareExchange( &perfstatsInit, 1, 0 ) )
			atexit( dumpPerfstats );
	}
#endif

	m_semaphore = CreateSemaphore( NULL, 0, 1, NULL );
	if( INVALID_HANDLE_VALUE != m_semaphore )
		return MRTE_RESULT_OK;
	else
		return MRTE_ERROR_LIBRARY_FAILURE;
}

